@@ -139,7 +139,7 @@ Job server database tables:
     workflow_id (optional)
     status
     client_paths
-    transfer_action_info
+    transfer_type
 
   Input/Ouput junction table
     job_id 
@@ -202,7 +202,7 @@ def create_database(database_file):
                                             workflow_id      INTEGER CONSTRAINT known_workflow REFERENCES workflows (id),
                                             status           VARCHAR(255) NOT NULL,
                                             client_paths     TEXT,
-                                            transfer_action_info  TEXT)''')
+                                            transfer_type TEXT)''')
 
   cursor.execute('''CREATE TABLE ios (job_id           INTEGER NOT NULL CONSTRAINT known_job REFERENCES jobs(id),
                                       engine_file_path  TEXT NOT NULL CONSTRAINT known_engine_file REFERENCES transfers (engine_file_path),
@@ -328,6 +328,7 @@ class WorkflowDatabaseServer( object ):
     @rtype: C{UserIdentifier}
     @return: user identifier
     '''
+    self.logger.debug("=> register_user")
     with self._lock:
       connection = self._connect()
       cursor = connection.cursor()
@@ -358,6 +359,7 @@ class WorkflowDatabaseServer( object ):
     Delete all expired jobs, transfers and workflows, except transfers which are requested 
     by valid job.
     '''
+    self.logger.debug("=> clean")
     self.remove_non_registered_files()
     with self._lock:
       connection = self._connect()
@@ -424,6 +426,7 @@ class WorkflowDatabaseServer( object ):
       connection.close()
      
   def remove_non_registered_files(self):
+    self.logger.debug("=> remove_non_registered_files")
     registered_engine_paths = []
     registered_users = []
     with self._lock:
@@ -463,7 +466,10 @@ class WorkflowDatabaseServer( object ):
   
      
      
-  def generate_file_path(self, user_id, client_file_path=None, external_cursor=None):
+  def generate_file_path(self, 
+                         user_id, 
+                         client_file_path=None, 
+                         external_cursor=None):
     '''
     Generates file path for transfers. 
     The user_id must be valid.
@@ -476,8 +482,10 @@ class WorkflowDatabaseServer( object ):
     @rtype: string
     @return: file path
     '''
+    
     with self._lock:
       if not external_cursor:
+        self.logger.debug("=> generate_file_path")
         connection = self._connect()
         cursor = connection.cursor()
       else:
@@ -544,12 +552,14 @@ class WorkflowDatabaseServer( object ):
     @type user_id:  C{UserIdentifier}
     @type  workflow_id: C{WorkflowIdentifier}
     '''
+    
     expiration_date = expiration_date
     if expiration_date == None:
       expiration_date = datetime.now() + timedelta(hours=engine_transfer.disposal_timeout) 
 
     with self._lock:
       if not external_cursor:
+        self.logger.debug("=> add_transfer")
         connection = self._connect()
         cursor = connection.cursor()
       else:
@@ -598,26 +608,6 @@ class WorkflowDatabaseServer( object ):
         connection.close()
 
     return engine_transfer
-    #return engine_transfer.engine_path
-
-
-  def is_valid_transfer(self, engine_file_path, user_id):
-    with self._lock:
-      connection = self._connect()
-      cursor = connection.cursor()
-      try:
-        count = cursor.execute('''SELECT count(*) 
-                                  FROM transfers 
-                                  WHERE engine_file_path=? and 
-                                        user_id=?''', 
-                                  [engine_file_path, user_id]).next()[0]
-      except Exception, e:
-        cursor.close()
-        connection.close()
-        raise DatabaseError('%s: %s \n' %(type(e), e))
-      cursor.close()
-      connection.close()
-    return count != 0
 
 
   def _check_transfer(self, connection, cursor, engine_file_path, user_id):
@@ -648,6 +638,7 @@ class WorkflowDatabaseServer( object ):
     @param engine_file_path: engine file path to identifying the transfer 
     record to delete.
     '''
+    self.logger.debug("=> remove_transfer")
     with self._lock:
       connection = self._connect()
       cursor = connection.cursor()
@@ -665,7 +656,9 @@ class WorkflowDatabaseServer( object ):
       connection.close()
       self.clean()
         
-  def get_transfer_information(self, engine_file_path):
+  def get_transfer_information(self, 
+                               engine_file_path, 
+                               user_id):
     '''
     Returns the information related to the transfer associated to the engine file path.
     The engine_file_path must be associated to a transfer.
@@ -673,46 +666,62 @@ class WorkflowDatabaseServer( object ):
     
     @type engine_file_path: string
     @rtype: tuple
-    @returns: (engine_file_path, client_file_path, expiration_date, workflow_id, client_paths)
+    @returns: (engine_file_path, client_file_path, expiration_date, workflow_id, client_paths, transfer_type, status)
     '''
+    self.logger.debug("=> get_transfer_information")
     with self._lock:
       connection = self._connect()
       cursor = connection.cursor()
+      self._check_transfer(connection, cursor, engine_file_path, user_id)
       try:
-        count = cursor.execute('SELECT count(*) FROM transfers WHERE engine_file_path=?', [engine_file_path]).next()[0]
-        if not count == 0 :
-          info = cursor.execute('''SELECT 
-                                   engine_file_path, 
-                                   client_file_path, 
-                                   expiration_date, 
-                                   workflow_id,
-                                   client_paths 
-                                   FROM transfers 
-                                   WHERE engine_file_path=?''', 
-                                   [engine_file_path]).next() #supposes that the engine_file_path is associated to a transfer
-        else: 
-          info = None
+        (engine_file_path,
+         client_file_path, 
+         expiration_date, 
+         workflow_id,
+         client_paths,
+         transfer_type,
+         status) = cursor.execute('''SELECT 
+                                  engine_file_path, 
+                                  client_file_path, 
+                                  expiration_date, 
+                                  workflow_id,
+                                  client_paths,
+                                  transfer_type,
+                                  status
+                                  FROM transfers 
+                                  WHERE engine_file_path=?''', 
+                                  [engine_file_path]).next() 
       except Exception, e:
         cursor.close()
         connection.close()
         raise DatabaseError('%s: %s \n' %(type(e), e)) 
-      if info:
-        if info[4]:
-          client_paths = self._string_conversion(info[4]).split(file_separator)
-        else: 
-          client_paths = None
-        result = (self._string_conversion(info[0]), self._string_conversion(info[1]), self._str_to_date_conversion(info[2]), info[3], client_paths)
+     
+      engine_file_path = self._string_conversion(engine_file_path)
+      client_file_path = self._string_conversion(client_file_path)
+      expiration_date = self._str_to_date_conversion(expiration_date)
+      if client_paths:
+        client_paths = self._string_conversion(client_paths).split(file_separator)
       else:
-        result = (None, None, None, -1, None)
+        client_path = None
+      transfer_type = self._string_conversion(transfer_type)
+      status = self._string_conversion(status)
+
       cursor.close()
       connection.close()
-    return result
+    return (engine_file_path, 
+            client_file_path, 
+            expiration_date, 
+            workflow_id, 
+            client_paths, 
+            transfer_type, 
+            status)
   
 
   def get_transfer_status(self, engine_file_path, user_id):
     '''
     Returns the transferstatus stored in the database.
     '''
+    self.logger.debug("=> get_transfer_status")
     with self._lock:
       connection = self._connect()
       cursor = connection.cursor()
@@ -739,6 +748,7 @@ class WorkflowDatabaseServer( object ):
     @type  status: string
     @param status: transfer status as defined in constants.FILE_TRANSFER_STATUS
     '''
+    self.logger.debug("=> set_transfer_status")
     with self._lock:
       # TBI if the status is not valid raise an exception ??
       connection = self._connect()
@@ -756,20 +766,14 @@ class WorkflowDatabaseServer( object ):
       cursor.close()
       connection.close()
 
-  def set_transfer_action_info(self, engine_file_path, info):
-    '''
-    Save data necessary for the transfer itself.
-    In the case of a file transfer, info is a tuple (file size, md5 hash)
-    In the case of a directory transfer, info is a tuple (cumulated file size, file transfer info)
-    '''
+
+  def set_transfer_type(self, engine_file_path, transfer_type, user_id):
+    self.logger.debug("=> set_transfer_type")
     with self._lock:
       connection = self._connect()
       cursor = connection.cursor()
       try:
-        count = cursor.execute('SELECT count(*) FROM transfers WHERE engine_file_path=?', [engine_file_path]).next()[0]
-        if not count == 0 :
-          pickled_info = pickle.dumps(info)
-          cursor.execute('UPDATE transfers SET transfer_action_info=? WHERE engine_file_path=?', (pickled_info, engine_file_path))
+        cursor.execute('UPDATE transfers SET transfer_type=? WHERE engine_file_path=?', (transfer_type, engine_file_path))
       except Exception, e:
         connection.rollback()
         cursor.close()
@@ -778,41 +782,14 @@ class WorkflowDatabaseServer( object ):
       connection.commit()
       cursor.close()
       connection.close()
-      
-      
-  def get_transfer_action_info(self, engine_file_path):
-    '''
-    Returns data necessary to the transfer itself.
-    '''
-    with self._lock:
-      connection = self._connect()
-      cursor = connection.cursor()
-      try:
-        count = cursor.execute('SELECT count(*) FROM transfers WHERE engine_file_path=?', [engine_file_path]).next()[0]
-        if not count == 0 :
-          pickled_info = cursor.execute('SELECT transfer_action_info FROM transfers WHERE engine_file_path=?', [engine_file_path]).next()[0]#supposes that the engine_file_path is associated to a transfer
-        else:
-          pickled_info = None
-      except Exception, e:
-        cursor.close()
-        connection.close()
-        raise DatabaseError('%s: %s \n' %(type(e), e))
-      
-      pickled_info = self._string_conversion(pickled_info)
-      if pickled_info:
-        info = pickle.loads(pickled_info)
-      else:
-        info = None
-      cursor.close()
-      connection.close()
- 
-    return info
+
 
   
   def add_workflow_ended_transfer(self, workflow_id, engine_file_path):
     '''
     To signal that a transfer belonging to a workflow finished.
     '''
+    self.logger.debug("=> add_workflow_ended_transfer")
     separator = ", "
     with self._lock:
       connection = self._connect()
@@ -842,6 +819,7 @@ class WorkflowDatabaseServer( object ):
     '''
     Returns the ended transfers for a workflow and clear the ended transfer list.
     '''
+    self.logger.debug("=> pop_workflow_ended_transfer")
     separator = ", "
     ended_transfers = []
     with self._lock:
@@ -885,6 +863,7 @@ class WorkflowDatabaseServer( object ):
           * dictionary job_id -> EngineJob
     '''
     # get back the workflow id first
+    self.logger.debug("=> add_workflow")
     with self._lock:
       connection = self._connect()
       cursor = connection.cursor()
@@ -954,6 +933,7 @@ class WorkflowDatabaseServer( object ):
     
     @type wf_id: C{WorkflowIdentifier}
     '''
+    self.logger.debug("=> delete_workflow")
     with self._lock:
       # set expiration date to yesterday + clean() ?
       connection = self._connect()
@@ -983,6 +963,7 @@ class WorkflowDatabaseServer( object ):
     @type wf_id: C{WorflowIdentifier}
     @type new_date: datetime.datetime
     '''
+    self.logger.debug("=> change_workflow_expiration_date")
     with self._lock:
       connection = self._connect()
       cursor = connection.cursor()
@@ -1007,6 +988,7 @@ class WorkflowDatabaseServer( object ):
     @rtype: C{EngineWorkflow}
     @return: workflow object
     '''
+    self.logger.debug("=> get_engine_workflow")
     with self._lock:
       connection = self._connect()
       cursor = connection.cursor()
@@ -1041,6 +1023,7 @@ class WorkflowDatabaseServer( object ):
     @type  status: string
     @param status: workflow status as defined in constants.WORKFLOW_STATUS
     '''
+    self.logger.debug("=> set_workflow_status")
     with self._lock:
       # TBI if the status is not valid raise an exception ??
       connection = self._connect()
@@ -1065,16 +1048,15 @@ class WorkflowDatabaseServer( object ):
                             datetime.now(), 
                             wf_id))
             if status == constants.WARNING:
-              print "WARNING"
               cursor.execute('''UPDATE jobs
                                 SET status=?
                                 WHERE workflow_id=? and 
                                       not status ISNULL and
                                       not status=? and not status=? ''',
                                 (constants.WARNING,
-                                 wf_id, 
-                                 constants.DONE, 
-                                 constants.FAILED))
+                                  wf_id, 
+                                  constants.DONE, 
+                                  constants.FAILED))
       except Exception, e:
         connection.rollback()
         cursor.close()
@@ -1089,6 +1071,7 @@ class WorkflowDatabaseServer( object ):
     Returns the workflow status stored in the database 
     (updated by L{DrmaaWorkflowEngine}) and the date of its last update.
     '''
+    self.logger.debug("=> get_workflow_status")
     with self._lock:
       connection = self._connect()
       cursor = connection.cursor()
@@ -1122,9 +1105,14 @@ class WorkflowDatabaseServer( object ):
                                       (submission_date, 
                                        execution_date, 
                                        ending_date)), 
-                   sequence of tuple (transfer_id, status), 
+                   sequence of tuple (transfer_id, 
+                                      client_file_path,
+                                      client_paths,
+                                      status,
+                                      transfer_type), 
                    workflow_status)
     '''
+    self.logger.debug("=> get_detailed_workflow_status")
     with self._lock:
       connection = self._connect()
       cursor = connection.cursor()
@@ -1160,22 +1148,30 @@ class WorkflowDatabaseServer( object ):
         # transfers 
         for row in cursor.execute('''SELECT engine_file_path, 
                                             client_file_path,
+                                            client_paths,
                                             status,
-                                            transfer_action_info
+                                            transfer_type
                                      FROM transfers WHERE workflow_id=?''', [wf_id]):
-          engine_file_path, client_file_path, status, pickled_info = row
+          (engine_file_path, 
+           client_file_path, 
+           client_paths, 
+           status, 
+           transfer_type) = row
             
           engine_file_path = self._string_conversion(engine_file_path)
           client_file_path = self._string_conversion(client_file_path)
           status = self._string_conversion(status)
-          
-          pickled_info = self._string_conversion(pickled_info)
-          if pickled_info:
-            transfer_action_info = pickle.loads(pickled_info)
+          transfer_type = self._string_conversion(transfer_type)
+          if client_paths:
+            client_paths = self._string_conversion(client_paths).split(file_separator)
           else:
-            transfer_action_info = None
+            client_paths = None
             
-          workflow_status[1].append((engine_file_path, client_file_path, status, transfer_action_info))
+          workflow_status[1].append((engine_file_path, 
+                                     client_file_path, 
+                                     client_paths, 
+                                     status,
+                                     transfer_type))
           
       except Exception, e:
         cursor.close()
@@ -1208,6 +1204,7 @@ class WorkflowDatabaseServer( object ):
                                 "user " + repr(user_id)) 
 
   def is_valid_job(self, job_id, user_id):
+    self.logger.debug("=> is_valid_job")
     with self._lock:
       connection = self._connect()
       cursor = connection.cursor()
@@ -1262,6 +1259,7 @@ class WorkflowDatabaseServer( object ):
 
     with self._lock:
       if not external_cursor:
+        self.logger.debug("=> add_job")
         connection = self._connect()
         cursor = connection.cursor()
       else:
@@ -1398,6 +1396,7 @@ class WorkflowDatabaseServer( object ):
     @rtype: C{EngineJob}
     @return: workflow object
     '''
+    self.logger.debug("=> get_engine_job")
     with self._lock:
       connection = self._connect()
       cursor = connection.cursor()
@@ -1430,6 +1429,7 @@ class WorkflowDatabaseServer( object ):
     
     @type job_id: 
     '''
+    self.logger.debug("=> delete_job")
     with self._lock:
       # set expiration date to yesterday + clean() ?
       connection = self._connect()
@@ -1452,6 +1452,56 @@ class WorkflowDatabaseServer( object ):
       self.clean()
 
 
+  def set_jobs_status(self, job_status, force=False):
+    '''
+    job_status: dictionary: job_id -> status
+    '''
+    self.logger.debug("=> set_jobs_status")
+    with self._lock:
+      # TBI if the status is not valid raise an exception ??
+      connection = self._connect()
+      cursor = connection.cursor()
+      try:
+        for job_id, status in job_status.iteritems():
+          count = cursor.execute('SELECT count(*) FROM jobs WHERE id=?', [job_id]).next()[0]
+          if not count == 0 :
+            (previous_status, 
+            execution_date, 
+            ending_date) = cursor.execute(''' SELECT status, 
+                                                      execution_date, 
+                                                      ending_date 
+                                              FROM jobs WHERE id=?''',
+                                              [job_id]).next()#supposes that the job_id is valid
+            previous_status = self._string_conversion(previous_status)
+            execution_date = self._str_to_date_conversion(execution_date)
+            ending_date = self._str_to_date_conversion(ending_date)
+            if previous_status != status:
+              if not execution_date and status == constants.RUNNING:
+                execution_date = datetime.now()
+              if not ending_date and status == constants.DONE or \
+                status == constants.FAILED:
+                ending_date = datetime.now()
+                if not execution_date :
+                  execution_date = datetime.now()
+            if force or \
+              (previous_status != constants.DELETE_PENDING and \
+                previous_status != constants.KILL_PENDING):
+              cursor.execute('''UPDATE jobs SET status=?, 
+                                                last_status_update=?,
+                                                execution_date=?, 
+                                                ending_date=? WHERE id=?''',
+                                                (status, datetime.now(),
+                                                execution_date, ending_date,
+                                                job_id))
+      except Exception, e:
+        connection.rollback()
+        cursor.close()
+        connection.close()
+        raise DatabaseError('%s: %s \n' %(type(e), e))
+      connection.commit()
+      cursor.close()
+      connection.close()
+
 
   def set_job_status(self, job_id, status, force = False):
     '''
@@ -1462,6 +1512,7 @@ class WorkflowDatabaseServer( object ):
     @type  status: string
     @param status: job status as defined in constants.JOB_STATUS
     '''
+    self.logger.debug("=> set_job_status")
     with self._lock:
       # TBI if the status is not valid raise an exception ??
       connection = self._connect()
@@ -1513,6 +1564,7 @@ class WorkflowDatabaseServer( object ):
     Raise UnknownObjectError if the job_id is not valid or belongs to an 
     other user.
     '''
+    self.logger.debug("=> get_job_status")
     with self._lock:
       connection = self._connect()
       cursor = connection.cursor()
@@ -1535,44 +1587,44 @@ class WorkflowDatabaseServer( object ):
  
     return (status, date)
 
-  def set_submission_information(self, job_id, drmaa_id, submission_date):
+  def set_submission_information(self, drmaa_ids, submission_date):
     '''
     Set the submission information of the job and reset information 
     related to the job submission (execution_date, ending_date, 
     exit_status, exit_value, terminating_signal, resource_usage) .
     
-    @type  drmaa_id: string
-    @param drmaa_id: job identifier on DRMS if submitted via DRMAA
-    @type  submission_date: date or None
-    @param submission_date: submission date if the job was submitted
+    *drmaa_ids: dictionary job_id -> drmaa_id
+    *submission_date: submission date if the job was submitted
     '''
+    self.logger.debug("=> set_submission_information")
     with self._lock:
       connection = self._connect()
       cursor = connection.cursor()
       try:
-        cursor.execute('''UPDATE jobs 
-                          SET drmaa_id=?, 
-                              submission_date=?, 
-                              status=?, 
-                              last_status_update=?,
-                              exit_status=?,
-                              exit_value=?,
-                              terminating_signal=?,
-                              resource_usage=?, 
-                              execution_date=?,
-                              ending_date=?
-                              WHERE id=?''', 
-                              (drmaa_id, 
-                              submission_date, 
-                              constants.UNDETERMINED, 
-                              datetime.now(),
-                              None,
-                              None,
-                              None,
-                              None,
-                              None,
-                              None,
-                              job_id))
+        for job_id, drmaa_id in drmaa_ids.iteritems():
+          cursor.execute('''UPDATE jobs 
+                            SET drmaa_id=?, 
+                                submission_date=?, 
+                                status=?, 
+                                last_status_update=?,
+                                exit_status=?,
+                                exit_value=?,
+                                terminating_signal=?,
+                                resource_usage=?, 
+                                execution_date=?,
+                                ending_date=?
+                                WHERE id=?''', 
+                                (drmaa_id, 
+                                submission_date, 
+                                constants.UNDETERMINED, 
+                                datetime.now(),
+                                None,
+                                None,
+                                None,
+                                None,
+                                None,
+                                None,
+                                job_id))
       except Exception, e:
         connection.rollback()
         cursor.close()
@@ -1591,6 +1643,7 @@ class WorkflowDatabaseServer( object ):
     @rtype: string
     @return: DRMAA job identifier (job identifier on DRMS if submitted via DRMAA)
     '''
+    self.logger.debug("=> get_drmaa_job_id")
     with self._lock:
       connection = self._connect()
       cursor = connection.cursor()
@@ -1617,6 +1670,7 @@ class WorkflowDatabaseServer( object ):
     @rtype: tuple
     @return: (stdout_file_path, stderr_file_path)
     '''
+    self.logger.debug("=> get_std_out_err_file_path")
     with self._lock:
       connection = self._connect()
       cursor = connection.cursor()
@@ -1632,6 +1686,12 @@ class WorkflowDatabaseServer( object ):
         connection.close()
         raise DatabaseError('%s: %s \n' %(type(e), e))
 
+
+      if count == 0:
+        raise UnknownObjectError("The job id " + repr(job_id) + " is not "
+                                 "valid or does not belong to "
+                                 "user " + repr(user_id)) 
+
       try:
         result = cursor.execute('SELECT stdout_file, stderr_file FROM jobs WHERE id=?', [job_id]).next()
       except Exception, e:
@@ -1652,6 +1712,7 @@ class WorkflowDatabaseServer( object ):
     @rtype: tuple
     @return: (exit_status, exit_value, terminating_signal, resource_usage)
     '''
+    self.logger.debug("=> get_job_exit_info")
     with self._lock:
       connection = self._connect()
       cursor = connection.cursor()
@@ -1673,11 +1734,39 @@ class WorkflowDatabaseServer( object ):
     exit_value = result[1]
     terminating_signal = self._string_conversion(result[2])
     resource_usage = self._string_conversion(result[3])
-
+ 
     return (exit_status, exit_value, terminating_signal, resource_usage)
 
 
-  def set_job_exit_info(self, job_id, exit_status, exit_value, terminating_signal, resource_usage):
+  def set_jobs_exit_info(self, job_dict):
+    self.logger.debug("=> set_jobs_exit_info")
+    with self._lock:
+      connection = self._connect()
+      cursor = connection.cursor()
+      try:
+        for job_id, job in job_dict.iteritems():
+          self.set_job_exit_info( job_id, 
+                                  job.exit_status, 
+                                  job.exit_value, 
+                                  job.terminating_signal, 
+                                  job.str_rusage,
+                                  cursor)
+      except Exception, e:
+        cursor.close()
+        connection.close()
+        raise DatabaseError('%s: %s \n' %(type(e), e)) 
+      connection.commit()
+      cursor.close()
+      connection.close()
+    
+
+  def set_job_exit_info(self, 
+                        job_id, 
+                        exit_status, 
+                        exit_value, 
+                        terminating_signal, 
+                        resource_usage, 
+                        external_cursor=None):
     '''
     Record the job exit status in the database.
     The status must be valid (ie a string among the exit job status 
@@ -1699,8 +1788,12 @@ class WorkflowDatabaseServer( object ):
     '''
     with self._lock:
       # TBI if the status is not valid raise an exception ??
-      connection = self._connect()
-      cursor = connection.cursor()
+      if not external_cursor:
+        self.logger.debug("=> set_job_exit_info")
+        connection = self._connect()
+        cursor = connection.cursor()
+      else:
+        cursor = external_cursor
       try:
         count = cursor.execute('SELECT count(*) FROM jobs WHERE id=?', [job_id]).next()[0]
         if not count == 0 :
@@ -1716,13 +1809,15 @@ class WorkflowDatabaseServer( object ):
                           job_id)
                         )
       except Exception, e:
-        connection.rollback()
+        if not external_cursor:
+          connection.rollback()
+          cursor.close()
+          connection.close()
+        raise DatabaseError('%s: %s \n' %(type(e), e)) 
+      if not external_cursor:
+        connection.commit()
         cursor.close()
         connection.close()
-        raise DatabaseError('%s: %s \n' %(type(e), e)) 
-      connection.commit()
-      cursor.close()
-      connection.close()
 
   def _string_conversion(self, string):
     if string: 
@@ -1751,6 +1846,7 @@ class WorkflowDatabaseServer( object ):
     @rtype: sequence of C{JobIdentifier}
     @returns: jobs owned by the user
     '''
+    self.logger.debug("=> get_jobs")
     if not job_ids:
       request = '''SELECT id, 
                           name, 
@@ -1800,6 +1896,7 @@ class WorkflowDatabaseServer( object ):
     @type queue_name: str
     @rtype: int
     '''
+    self.logger.debug("=> nb_queued_jobs")
     with self._lock:
       connection = self._connect()
       cursor = connection.cursor()
@@ -1837,6 +1934,7 @@ class WorkflowDatabaseServer( object ):
     @rtype: sequence of C{JobIdentifier}
     @returns: job with status constants.DELETE_PENDING
     '''
+    self.logger.debug("=> jobs_to_delete_and_kill")
     with self._lock:
       connection = self._connect()
       cursor = connection.cursor()
@@ -1876,6 +1974,7 @@ class WorkflowDatabaseServer( object ):
     @rtype: sequence of engine file path
     @returns: engine file path associated with a transfer owned by the user
     '''
+    self.logger.debug("=> get_transfers")
     if not transfer_ids:
       request = '''SELECT engine_file_path,
                           client_file_path, 
@@ -1940,6 +2039,7 @@ class WorkflowDatabaseServer( object ):
 
 
   def is_valid_workflow(self, wf_id, user_id):
+    self.logger.debug("=> is_valid_workflow")
     with self._lock:
       connection = self._connect()
       cursor = connection.cursor()
@@ -1977,6 +2077,7 @@ class WorkflowDatabaseServer( object ):
     @type user_id: C{UserIdentifier}
     @rtype: sequence of workflows id
     '''
+    self.logger.debug("=> get_workflows")
     if not workflow_ids:
       request = "SELECT id, name, expiration_date FROM workflows WHERE user_id=?"
       argument = [user_id]
@@ -2015,6 +2116,7 @@ class WorkflowDatabaseServer( object ):
     @rtype: sequence of C{WorkflowIdentifier}
     @returns: workflows with status constants.DELETE_PENDING
     '''
+    self.logger.debug("=> workflows_to_delete_and_kill")
     with self._lock:
       connection = self._connect()
       cursor = connection.cursor()
