@@ -124,6 +124,83 @@ class EngineJob(Job):
         raise JobError("The parallel job can not be submitted because the "
                         "parallel configuration %s is missing." %(configuration_name))
 
+
+    if self.stdin:
+      if isinstance(self.stdin, FileTransfer):
+        if not self.stdin in self.referenced_input_files:
+          self.referenced_input_files.append(self.stdin)
+        #if not self.stdin in self.transfer_mapping:
+          #if isinstance(self.stdin, EngineTransfer):
+            ## TBI check that the transfer exist in the database 
+            #self.transfer_mapping[self.stdin] = self.stdin
+          #else:
+            #eft = EngineTransfer(self.stdin)
+            #self.transfer_mapping[self.stdin] = eft
+      elif isinstance(self.stdin, SharedResourcePath):
+        self.srp_mapping[self.stdin] = self._translate(self.stdin) 
+      else:
+        if not type(self.stdin) in types.StringTypes:
+          raise JobError("Wrong stdin type: %s" %(repr(self.stdin))) 
+        self.stdin = os.path.abspath(self.stdin)
+
+    if self.working_directory:
+      if isinstance(self.working_directory, FileTransfer):
+        if not self.working_directory in self.referenced_input_files:
+          self.referenced_input_files.append(self.working_directory)
+        if not self.working_directory in self.referenced_output_files:
+          self.referenced_output_files.append(self.working_directory)
+        #if not self.working_directory in self.transfer_mapping:
+          #if isinstance(self.working_directory, EngineTransfer):
+            ## TBI check that the transfer exist in the database 
+            #self.transfer_mapping[self.working_directory] = self.working_directory
+          #else:
+            #eft = EngineTransfer(self.working_directory)
+            #self.transfer_mapping[self.working_directory] = eft
+      elif isinstance(self.working_directory, SharedResourcePath):
+        self.srp_mapping[self.working_directory] = self._translate(self.working_directory)
+      else:
+        if not type(self.working_directory) in types.StringTypes:
+          raise JobError("Wrong working directory type: %s " %
+                         (repr(self.working_directory)))
+        self.working_directory = os.path.abspath(self.working_directory)
+
+    if self.stdout_file:
+      if isinstance(self.stdout_file, FileTransfer):
+        if not self.stdout_file in self.referenced_output_files:
+          self.referenced_output_files.append(self.stdout_file)
+        #if not self.stdout_file in self.transfer_mapping:
+          #if isinstance(self.stdout_file, EngineTransfer):
+            ## TBI check that the transfer exist in the database 
+            #self.transfer_mapping[self.stdout_file] = self.stdout_file
+          #else:
+            #eft = EngineTransfer(self.stdout_file)
+            #self.transfer_mapping[self.stdout_file] = eft
+      elif isinstance(self.stdout_file, SharedResourcePath):
+        self.srp_mapping[self.stdout_file] = self._translate(self.stdout_file) 
+      else:
+        if not type(self.stdout_file) in types.StringTypes:
+          raise JobError("Wrong stdout_file type: %s" %(repr(self.stdout_file))) 
+        self.stdout_file = os.path.abspath(self.stdout_file)
+
+    if self.stderr_file:
+      if isinstance(self.stderr_file, FileTransfer):
+        if not self.stderr_file in self.referenced_output_files:
+          self.referenced_output_files.append(self.stderr_file)
+        #if not self.stderr_file in self.transfer_mapping:
+          #if isinstance(self.stderr_file, EngineTransfer):
+            ## TBI check that the transfer exist in the database 
+            #self.transfer_mapping[self.stderr_file] = self.stderr_file
+          #else:
+            #eft = EngineTransfer(self.stderr_file)
+            #self.transfer_mapping[self.stderr_file] = eft
+      elif isinstance(self.stderr_file, SharedResourcePath):
+        self.srp_mapping[self.stderr_file] = self._translate(self.stderr_file) 
+      else:
+        if not type(self.stderr_file) in types.StringTypes:
+          raise JobError("Wrong stderr_file type: %s" %(repr(self.stderr_file))) 
+        self.stderr_file = os.path.abspath(self.stderr_file)
+    
+
     # transfer_mapping from referenced_input_files and referenced_output_files
     # + type checking
     for ft in self.referenced_input_files:
@@ -195,71 +272,7 @@ class EngineJob(Job):
         if not type(command_el) in types.StringTypes:
           raise JobError("Wrong command element type: %s" %(repr(command_el)))
       
-    if self.stdin:
-      if isinstance(self.stdin, FileTransfer):
-        if not self.stdin in self.transfer_mapping:
-          self.referenced_input_files.append(self.stdin)
-          if isinstance(self.stdin, EngineTransfer):
-            # TBI check that the transfer exist in the database 
-            self.transfer_mapping[self.stdin] = self.stdin
-          else:
-            eft = EngineTransfer(self.stdin)
-            self.transfer_mapping[self.stdin] = eft
-      elif isinstance(self.stdin, SharedResourcePath):
-        self.srp_mapping[self.stdin] = self._translate(self.stdin) 
-      else:
-        if not type(self.stdin) in types.StringTypes:
-          raise JobError("Wrong stdin type: %s" %(repr(self.stdin))) 
-
-    if self.working_directory:
-      if isinstance(self.working_directory, FileTransfer):
-        if not self.working_directory in self.transfer_mapping:
-          self.referenced_input_files.append(self.working_directory)
-          self.referenced_output_files.append(self.working_directory)
-          if isinstance(self.working_directory, EngineTransfer):
-            # TBI check that the transfer exist in the database 
-            self.transfer_mapping[self.working_directory] = self.working_directory
-          else:
-            eft = EngineTransfer(self.working_directory)
-            self.transfer_mapping[self.working_directory] = eft
-      elif isinstance(self.working_directory, SharedResourcePath):
-        self.srp_mapping[self.working_directory] = self._translate(self.working_directory)
-      else:
-        if not type(self.working_directory) in types.StringTypes:
-          raise JobError("Wrong working directory type: %s " %
-                         (repr(self.working_directory)))
-
-    if self.stdout_file:
-      if isinstance(self.stdout_file, FileTransfer):
-        if not self.stdout_file in self.transfer_mapping:
-          self.referenced_output_files.append(self.stdout_file)
-          if isinstance(self.stdout_file, EngineTransfer):
-            # TBI check that the transfer exist in the database 
-            self.transfer_mapping[self.stdout_file] = self.stdout_file
-          else:
-            eft = EngineTransfer(self.stdout_file)
-            self.transfer_mapping[self.stdout_file] = eft
-      elif isinstance(self.stdout_file, SharedResourcePath):
-        self.srp_mapping[self.stdout_file] = self._translate(self.stdout_file) 
-      else:
-        if not type(self.stdout_file) in types.StringTypes:
-          raise JobError("Wrong stdout_file type: %s" %(repr(self.stdout_file))) 
-
-    if self.stderr_file:
-      if isinstance(self.stderr_file, FileTransfer):
-        if not self.stderr_file in self.transfer_mapping:
-          self.referenced_output_files.append(self.stderr_file)
-          if isinstance(self.stderr_file, EngineTransfer):
-            # TBI check that the transfer exist in the database 
-            self.transfer_mapping[self.stderr_file] = self.stderr_file
-          else:
-            eft = EngineTransfer(self.stderr_file)
-            self.transfer_mapping[self.stderr_file] = eft
-      elif isinstance(self.stderr_file, SharedResourcePath):
-        self.srp_mapping[self.stderr_file] = self._translate(self.stderr_file) 
-      else:
-        if not type(self.stderr_file) in types.StringTypes:
-          raise JobError("Wrong stderr_file type: %s" %(repr(self.stderr_file))) 
+ 
 
   def _translate(self, srp):
     '''
@@ -268,15 +281,14 @@ class EngineJob(Job):
     '''
     
     if not self.path_translation:
-      raise JobError("The job uses SharedResourcePath while no "
-                     "translation were found.")
+      raise JobError("Could not submit workflows or jobs with shared resource "
+                     "path: no translation found in the configuration file.")
     if not srp.namespace in self.path_translation.keys():
-      raise JobError("SharedResourcePath translation: the "               
-                     "namespace %s does not exist" %(srp.namespace))
+      raise JobError("Could not translate shared resource path. The "               
+                     "namespace %s is not configured." %(srp.namespace))
     if not srp.uuid in self.path_translation[srp.namespace]:
-      raise JobError("SharedResourcePath translation: "
-                     "the uuid %s does not exist for the " 
-                     "namespace %s." %
+      raise JobError("Could not translate shared resource path. The uuid %s "
+                     "does not exist for the namespace %s." %
                      (srp.uuid, srp.namespace))
     translated_path = os.path.join(self.path_translation[srp.namespace][srp.uuid],
                                    srp.relative_path)
@@ -417,7 +429,7 @@ class EngineWorkflow(Workflow):
   # dictonary: tr_id -> EngineTransfer
   registered_tr = None
 
-  #logger = None
+  logger = None
   
   def __init__(self, 
                client_workflow, 
@@ -476,7 +488,7 @@ class EngineWorkflow(Workflow):
         self.jobs.append(dependency[0])
         ejob = EngineJob( client_job=dependency[0],
                           queue=self.queue,
-                          path_translation=self.path_translation,
+                          path_translation=self._path_translation,
                           transfer_mapping=self.transfer_mapping)
         self.transfer_mapping.update(ejob.transfer_mapping)
         self.job_mapping[dependency[0]]=ejob
@@ -485,7 +497,7 @@ class EngineWorkflow(Workflow):
         self.jobs.append(dependency[1])
         ejob = EngineJob( client_job=dependency[1],
                           queue=self.queue,
-                          path_translation=self.path_translation,
+                          path_translation=self._path_translation,
                           transfer_mapping=self.transfer_mapping)
         self.transfer_mapping.update(ejob.transfer_mapping)
         self.job_mapping[dependency[1]]=ejob
@@ -499,7 +511,7 @@ class EngineWorkflow(Workflow):
             self.jobs.append(elem)
             ejob = EngineJob( client_job=elem,
                               queue=self.queue,
-                              path_translation=self.path_translation,
+                              path_translation=self._path_translation,
                               transfer_mapping=self.transfer_mapping)
             self.transfer_mapping.update(ejob.transfer_mapping)
             self.job_mapping[elem]=ejob
@@ -515,7 +527,7 @@ class EngineWorkflow(Workflow):
           self.jobs.append(elem)
           ejob = EngineJob( client_job=elem,
                             queue=self.queue,
-                            path_translation=self.path_translation,
+                            path_translation=self._path_translation,
                             transfer_mapping=self.transfer_mapping)
           self.transfer_mapping.update(ejob.transfer_mapping)
           self.job_mapping[elem]=ejob
@@ -561,7 +573,7 @@ class EngineWorkflow(Workflow):
               workflow status)
     '''
 
-    logger = logging.getLogger('engine.EngineWorkflow') 
+    self.logger = logging.getLogger('engine.EngineWorkflow') 
     to_run = []
     to_abort = set([])
     done = []
@@ -572,7 +584,7 @@ class EngineWorkflow(Workflow):
         done.append(job)
       elif job.is_running(): 
         running.append(job)
-      logger.debug("job " + repr(job.name) + " " + repr(job.status) + " r " + repr(job.is_running()) + " d " + repr(job.is_done()))
+      self.logger.debug("job " + repr(job.name) + " " + repr(job.status) + " r " + repr(job.is_running()) + " d " + repr(job.is_done()))
       if job.status == constants.NOT_SUBMITTED:
         # a job can start to run when all its dependencies succeed and 
         # all its input files are in the FILES_ON_CR or 
@@ -598,11 +610,11 @@ class EngineWorkflow(Workflow):
           wf_running = True
           to_run.append(job)
 
-    logger.debug(" ")
-    logger.debug("to run " + repr(to_run))
-    logger.debug("to abort " + repr(to_abort))
-    logger.debug("done " + repr(done))
-    logger.debug("running " + repr(running))
+    #self.logger.debug(" ")
+    #self.logger.debug("to run " + repr(to_run))
+    #self.logger.debug("to abort " + repr(to_abort))
+    #self.logger.debug("done " + repr(done))
+    #self.logger.debug("running " + repr(running))
 
     # if a job fails the whole workflow branch has to be stopped
     # look for the node in the branch to abort
@@ -619,9 +631,9 @@ class EngineWorkflow(Workflow):
     # stop the whole branch
     ended_jobs = {}
     for job in to_abort:
-      if job.job_id:
-        #self.logger.debug("  ---- Failure: job to abort " + job.name)
-        assert(job.status == constants.NOT_SUBMITTED)
+      if job.job_id and job.status != constants.FAILED:
+        #self.self.logger.debug("  ---- Failure: job to abort " + job.name)
+        assert(job.status == constants.NOT_SUBMITTED or job.status == constants.WARNING)
         ended_jobs[job.job_id] = job
         job.status = constants.FAILED
         job.exit_status = constants.EXIT_ABORTED
@@ -635,7 +647,7 @@ class EngineWorkflow(Workflow):
       status = constants.WORKFLOW_IN_PROGRESS
       # !!!! the workflow may be stuck !!!!
       # TBI
-      logger.debug("!!!! The workflow may be stuck !!!!")
+      self.logger.debug("!!!! The workflow may be stuck !!!!")
     else:
       status = constants.WORKFLOW_NOT_STARTED
 
@@ -658,11 +670,18 @@ class EngineWorkflow(Workflow):
       self.registered_jobs[job_id].terminating_signal = term_signal
    
     for ft_info in wf_status[1]:
-      engine_path, client_path, status, transfer_action_info = ft_info 
+      (engine_path, 
+       client_path, 
+       client_paths,
+       status, 
+       transfer_type) = ft_info 
       self.registered_tr[engine_path].status = status
 
     done = True
-    for job in self.registered_jobs.itervalues():
+    sub_info_to_resert = {}
+    new_status = {}
+    for client_job in self.jobs:
+      job = self.job_mapping[client_job]
       if job.failed():
         #clear all the information related to the previous job submission
         job.status = constants.NOT_SUBMITTED
@@ -674,11 +693,16 @@ class EngineWorkflow(Workflow):
         stdout.close()
         stderr = open(job.stderr_file, "w")
         stderr.close()
-        database_server.set_submission_information(job.job_id, None, None)
-        database_server.set_job_status(job.job_id, constants.NOT_SUBMITTED)
+
+        sub_info_to_resert[job.job_id] = None
+        new_status[job.job_id] = constants.NOT_SUBMITTED
+
       if not job.ended_with_success():
         undone_jobs.append(job)
 
+    database_server.set_submission_information(sub_info_to_resert, None)
+    database_server.set_jobs_status(new_status)
+
     to_run = []
     if undone_jobs:
       # look for jobs to run
@@ -743,4 +767,4 @@ class EngineTransfer(FileTransfer):
             self.status == constants.FILES_ON_CLIENT_AND_CR or \
             self.status == constants.TRANSFERING_FROM_CR_TO_CLIENT
     return exist
- 
\ No newline at end of file
+ 
