@@ -23,19 +23,25 @@ import matplotlib
 matplotlib.use('Qt4Agg')
 from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
 from matplotlib.figure import Figure 
+import socket
 
 from soma.workflow.client import Workflow, Group, FileTransfer, SharedResourcePath, Job, WorkflowController, Helper
 from soma.workflow.engine_types import EngineWorkflow, EngineJob, EngineTransfer
 from soma.workflow.constants import *
 from soma.workflow.configuration import Configuration
 from soma.workflow.test.test_workflow import WorkflowExamples
-from soma.workflow.errors import UnknownObjectError, ConfigurationError, SerializationError
+from soma.workflow.errors import UnknownObjectError, ConfigurationError, SerializationError, WorkflowError, JobError
+import soma.workflow.utils
 
 
-class PyroError(Exception):     pass
-class ProtocolError(PyroError): pass
-class ConnectionClosedError(ProtocolError): pass
 
+try:
+  from Pyro.errors import ConnectionClosedError
+except ImportError:
+  # Pyro is not required when soma-workflow runs as a one process application
+  class PyroError(Exception):     pass
+  class ProtocolError(PyroError): pass
+  class ConnectionClosedError(ProtocolError): pass
 
 #-----------------------------------------------------------------------------
 # Globals and constants
@@ -136,8 +142,14 @@ def setLabelFromDateTime(label, value):
 class Controller(object):
   @staticmethod
   def delete_workflow(wf_id, 
+                      force, 
                       wf_ctrl):
-    return wf_ctrl.delete_workflow(wf_id)
+    return wf_ctrl.delete_workflow(wf_id, force)
+
+  @staticmethod
+  def stop_workflow(wf_id, 
+                    wf_ctrl):
+    return wf_ctrl.stop_workflow(wf_id)
   
   @staticmethod
   def change_workflow_expiration_date(wf_id, 
@@ -149,19 +161,6 @@ class Controller(object):
   def get_submitted_workflows(wf_ctrl):
     return wf_ctrl.workflows()
 
-  @staticmethod
-  def submit_workflow(workflow, 
-                      name, 
-                      expiration_date,
-                      queue,
-                      wf_ctrl):
-    wf_id = wf_ctrl.submit_workflow(workflow=workflow,
-                                    expiration_date=expiration_date,
-                                    name=name,
-                                    queue=queue) 
-    wf = wf_ctrl.workflow(wf_id)
-    return wf
-
   @staticmethod
   def restart_workflow(workflow_id, wf_ctrl):
     return wf_ctrl.restart_workflow(workflow_id)
@@ -208,15 +207,21 @@ class Controller(object):
   def transfer_output_files(workflow, wf_ctrl, buffer_size= 256**2):
     Helper.transfer_output_files(workflow, wf_ctrl, buffer_size)
 
+  @staticmethod
+  def optimize_workflow(workflow):
+    return soma.workflow.utils.optimize_workflow(workflow)
+
 
 class WorkflowWidget(QtGui.QMainWindow):
   
-  def __init__(self, model, parent=None, flags=0):
+  def __init__(self, model, user=None, auto_connect=False, parent=None, flags=0):
     super(WorkflowWidget, self).__init__(parent)
     
     self.ui = Ui_WorkflowMainWindow()
     self.ui.setupUi(self)
     
+    self.setWindowIcon(QtGui.QIcon(os.path.join(os.path.dirname(__file__), "icon/soma_workflow_icon.png")))
+
     self.setCorner(QtCore.Qt.BottomLeftCorner, QtCore.Qt.LeftDockWidgetArea)
     self.setCorner(QtCore.Qt.BottomRightCorner, QtCore.Qt.RightDockWidgetArea)
     
@@ -285,11 +290,13 @@ class WorkflowWidget(QtGui.QMainWindow):
     self.ui.action_transfer_infiles.triggered.connect(self.transferInputFiles)
     self.ui.action_transfer_outfiles.triggered.connect(self.transferOutputFiles)
     self.ui.action_open_wf.triggered.connect(self.openWorkflow)
+    self.ui.action_optimize_wf.triggered.connect(self.optimize_workflow)
     self.ui.action_create_wf_ex.triggered.connect(self.createWorkflowExample)
     self.ui.action_delete_workflow.triggered.connect(self.delete_workflow)
-    self.ui.action_change_expiration_date.triggered.connect(self.changeExpirationDate)
+    self.ui.action_change_expiration_date.triggered.connect(self.change_expiration_date)
     self.ui.action_save.triggered.connect(self.saveWorkflow)
     self.ui.action_restart.triggered.connect(self.restart_workflow)
+    self.ui.action_stop_wf.triggered.connect(self.stop_workflow)
     
     self.ui.list_widget_submitted_wfs.itemSelectionChanged.connect(self.workflowSelectionChanged)
     self.ui.combo_resources.currentIndexChanged.connect(self.resourceSelectionChanged)
@@ -297,19 +304,57 @@ class WorkflowWidget(QtGui.QMainWindow):
     self.ui.wf_list_refresh_button.clicked.connect(self.refreshWorkflowList)
     
     self.showMaximized()
-    
-    # first connection:
+
     self.firstConnection_dlg = QtGui.QDialog(self)
     self.ui_firstConnection_dlg = Ui_FirstConnectionDlg()
     self.ui_firstConnection_dlg.setupUi(self.firstConnection_dlg)
     self.ui_firstConnection_dlg.combo_resources.addItems(self.resource_list)
     self.firstConnection_dlg.accepted.connect(self.firstConnection)
     self.firstConnection_dlg.rejected.connect(self.close)
-    self.firstConnection_dlg.show()
+    
+    ## First connection:
+
+    # Try to connect directly:  
+    if auto_connect or self.config_file_path == None:
+      if user is not None and len(self.resource_list) > 0:
+        self.connect_to_controller(self.resource_list[0], user)
+      else:
+        self.connect_to_controller(socket.gethostname())
+    else: # Show connection dialog:
+      if user is not None:
+        self.ui_firstConnection_dlg.lineEdit_login.setText(user)
+      self.firstConnection_dlg.show()
     
     if not self.model.current_connection:
       self.close()
-      
+
+
+    self.currentWorkflowChanged()
+
+  def connect_to_controller(self, 
+                            resource_id, 
+                            login=None, 
+                            password=None, 
+                            rsa_key_pass=None):      
+
+    wf_ctrl = None
+    try:
+      wf_ctrl = Controller.get_connection(resource_id, 
+                                          login, 
+                                          password,
+                                          rsa_key_pass)
+    except ConfigurationError, e:
+      QtGui.QMessageBox.critical(self, "Configuration problem", "%s" %(e))
+      self.ui_firstConnection_dlg.lineEdit_password.clear()
+      self.firstConnection_dlg.show()
+    except Exception, e:
+      QtGui.QMessageBox.critical(self, "Connection failed", "%s" %(e))
+      self.ui_firstConnection_dlg.lineEdit_password.clear()
+      self.firstConnection_dlg.show()
+    else:
+      self.model.add_connection(resource_id, wf_ctrl)
+      self.firstConnection_dlg.hide()
+
       
   @QtCore.pyqtSlot()
   def firstConnection(self):
@@ -327,23 +372,8 @@ class WorkflowWidget(QtGui.QMainWindow):
     else:
       rsa_key_pass = None
 
-    wf_ctrl = None
-    try:
-      wf_ctrl = Controller.get_connection(resource_id, 
-                                          login, 
-                                          password,
-                                          rsa_key_pass)
-    except ConfigurationError, e:
-      QtGui.QMessageBox.critical(self, "Configuration problem", "%s" %(e))
-      self.ui_firstConnection_dlg.lineEdit_password.clear()
-      self.firstConnection_dlg.show()
-    except Exception, e:
-      QtGui.QMessageBox.critical(self, "Connection failed", "%s" %(e))
-      self.ui_firstConnection_dlg.lineEdit_password.clear()
-      self.firstConnection_dlg.show()
-    else:
-      self.model.addConnection(resource_id, wf_ctrl)
-      self.firstConnection_dlg.hide()
+    self.connect_to_controller(resource_id, login, password, rsa_key_pass)
+    
     
   @QtCore.pyqtSlot()
   def openWorkflow(self):
@@ -355,7 +385,18 @@ class WorkflowWidget(QtGui.QMainWindow):
         QtGui.QMessageBox.warning(self, "Error opening the workflow", "%s" %(e))
       else:
         self.updateWorkflowList()
-        self.model.addWorkflow(workflow, datetime.now() + timedelta(days=5))
+        self.model.add_workflow(workflow, datetime.now() + timedelta(days=5))
+
+  @QtCore.pyqtSlot()
+  def optimize_workflow(self):
+    assert(self.model.current_workflow)
+    assert(self.model.current_workflow.wf_id == -1)
+  
+    new_workflow = Controller.optimize_workflow(self.model.current_workflow.server_workflow)
+
+    self.updateWorkflowList()
+    self.model.add_workflow(new_workflow, datetime.now() + timedelta(days=5))
+    
       
   @QtCore.pyqtSlot()
   def saveWorkflow(self):
@@ -424,13 +465,23 @@ class WorkflowWidget(QtGui.QMainWindow):
                             name, 
                             queue, 
                             self.model.current_connection)
+        except WorkflowError, e:
+          QtGui.QMessageBox.warning(self, 
+                                "Workflow submission error", 
+                                "%s" %(e))
+          return
+        except JobError, e:
+          QtGui.QMessageBox.warning(self, 
+                                "Workflow submission error", 
+                                "%s" %(e))
+          return
         except ConnectionClosedError, e:
           if not self.reconnectAfterConnectionClosed():
             return
         else:
           break
       self.updateWorkflowList()
-      self.model.addWorkflow(workflow, date) 
+      self.model.add_workflow(workflow, date) 
 
   @QtCore.pyqtSlot()
   def restart_workflow(self):
@@ -446,7 +497,7 @@ class WorkflowWidget(QtGui.QMainWindow):
                                 "Restart workflow", 
                                 "The workflow is already running.")
     else:
-      self.model.restartCurrentWorkflow()
+      self.model.restart_current_workflow()
   
   @QtCore.pyqtSlot()
   def transferInputFiles(self):
@@ -496,8 +547,8 @@ class WorkflowWidget(QtGui.QMainWindow):
       return
     wf_id = selected_items[0].data(QtCore.Qt.UserRole).toInt()[0]
     if wf_id != -1:
-      if self.model.isLoadedWorkflow(wf_id):
-        self.model.setCurrentWorkflow(wf_id)
+      if self.model.is_loaded_workflow(wf_id):
+        self.model.set_current_workflow(wf_id)
       else:
         workflow = self.model.current_connection.workflow(wf_id)
         if workflow:
@@ -505,12 +556,12 @@ class WorkflowWidget(QtGui.QMainWindow):
         else:
           expiration_date = None
         if workflow != None:
-          self.model.addWorkflow(workflow, expiration_date)
+          self.model.add_workflow(workflow, expiration_date)
         else:
           self.updateWorkflowList()
-          self.model.clearCurrentWorkflow()
+          self.model.clear_current_workflow()
     else:
-      self.model.clearCurrentWorkflow()
+      self.model.clear_current_workflow()
     
   @QtCore.pyqtSlot(int)
   def resourceSelectionChanged(self, index):
@@ -525,13 +576,13 @@ class WorkflowWidget(QtGui.QMainWindow):
       self.ui.combo_resources.setCurrentIndex(index)
       return
     
-    if resource_id in self.model.connections.keys():
-      self.model.setCurrentConnection(resource_id)
+    if self.model.resource_exist(resource_id):
+      self.model.set_current_connection(resource_id)
       return
     else:
       new_connection = self.createConnection(resource_id)
       if new_connection:
-        self.model.addConnection(resource_id, new_connection)
+        self.model.add_connection(resource_id, new_connection)
 
   def createConnection(self, resource_id):
     connection_invalid = True
@@ -573,6 +624,27 @@ class WorkflowWidget(QtGui.QMainWindow):
         return wf_ctrl
     return None
 
+  @QtCore.pyqtSlot()
+  def stop_workflow(self):
+    assert(self.model.current_workflow and self.model.current_wf_id != -1)
+    
+    if self.model.current_workflow.name:
+      name = self.model.current_workflow.name
+    else: 
+      name = repr(self.model.current_wf_id)
+    
+    answer = QtGui.QMessageBox.question(self, "confirmation", "The running jobs will be killed and the jobs in the queue will be removed. \nDo you want to stop the workflow " + name +" anyway?", QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
+    if answer != QtGui.QMessageBox.Yes: return
+    while True:
+      try: 
+        Controller.stop_workflow(self.model.current_workflow.wf_id,
+                                 self.model.current_connection)
+      except ConnectionClosedError, e:
+        if not self.reconnectAfterConnectionClosed():
+          return
+      else:
+        break
+
   @QtCore.pyqtSlot()
   def delete_workflow(self):
     assert(self.model.current_workflow and self.model.current_wf_id != -1)
@@ -582,22 +654,35 @@ class WorkflowWidget(QtGui.QMainWindow):
     else: 
       name = repr(self.model.current_wf_id)
     
+    
     answer = QtGui.QMessageBox.question(self, "confirmation", "Do you want to delete the workflow " + name +"?", QtGui.QMessageBox.Yes, QtGui.QMessageBox.No)
     if answer != QtGui.QMessageBox.Yes: return
+    force = self.ui.check_box_force_delete.isChecked()
     while True:
       try:
-        Controller.delete_workflow(self.model.current_workflow.wf_id,
+        deleled_properly = Controller.delete_workflow(self.model.current_workflow.wf_id,
+                                   force,
                                    self.model.current_connection)
       except ConnectionClosedError, e:
         if not self.reconnectAfterConnectionClosed():
           return
       else:
         break
-    self.updateWorkflowList()
-    self.model.delete_workflow()
+    if force:
+      self.updateWorkflowList()
+      self.model.delete_workflow()
+      if not deleled_properly:
+         QtGui.QMessageBox.warning(self, 
+                                   "Delete workflow", 
+                                   "The workflow was deleted. However, "
+                                   "it possible that some workflow jobs are "
+                                   "still active and burden the computing "
+                                   "resource. Please inspect the active jobs "
+                                   "(running or in the queue) using the DRMS " "interface.")
+        
     
   @QtCore.pyqtSlot()
-  def changeExpirationDate(self):
+  def change_expiration_date(self):
     qtdt = self.ui.dateTimeEdit_expiration.dateTime()
     date = datetime(qtdt.date().year(), qtdt.date().month(), qtdt.date().day(), 
                     qtdt.time().hour(), qtdt.time().minute(), qtdt.time().second())
@@ -617,14 +702,14 @@ class WorkflowWidget(QtGui.QMainWindow):
       QtGui.QMessageBox.information(self, "information", "The workflow expiration date was not changed.")
       self.ui.dateTimeEdit_expiration.setDateTime(self.expiration_date)
     else:
-      self.model.changeExpirationDate(date)
+      self.model.change_expiration_date(date)
       
   
   @QtCore.pyqtSlot()
   def currentConnectionChanged(self):
     self.setWindowTitle("soma-workflow - " + self.model.current_resource_id)
     self.updateWorkflowList()
-    self.model.clearCurrentWorkflow()
+    self.model.clear_current_workflow()
     index = self.ui.combo_resources.findText(self.model.current_resource_id)
     self.ui.combo_resources.setCurrentIndex(index)
   
@@ -646,8 +731,12 @@ class WorkflowWidget(QtGui.QMainWindow):
       self.ui.dateTimeEdit_expiration.setDateTime(datetime.now())
       self.ui.dateTimeEdit_expiration.setEnabled(False)
       
+      self.ui.action_optimize_wf.setEnabled(False)
+
       self.ui.action_submit.setEnabled(False)
       self.ui.action_change_expiration_date.setEnabled(False)
+      self.ui.action_stop_wf.setEnabled(False)
+      self.ui.action_restart.setEnabled(False)
       self.ui.action_delete_workflow.setEnabled(False)
       self.ui.action_transfer_infiles.setEnabled(False)
       self.ui.action_transfer_outfiles.setEnabled(False)
@@ -677,9 +766,12 @@ class WorkflowWidget(QtGui.QMainWindow):
         self.ui.dateTimeEdit_expiration.setDateTime(datetime.now() + timedelta(days=5))
         self.ui.dateTimeEdit_expiration.setEnabled(True)
         
+        self.ui.action_optimize_wf.setEnabled(True)
         self.ui.action_submit.setEnabled(True)
         self.ui.action_change_expiration_date.setEnabled(False)
         self.ui.action_delete_workflow.setEnabled(False)
+        self.ui.action_stop_wf.setEnabled(False)
+        self.ui.action_restart.setEnabled(False)
         self.ui.action_transfer_infiles.setEnabled(False)
         self.ui.action_transfer_outfiles.setEnabled(False)
         
@@ -700,9 +792,12 @@ class WorkflowWidget(QtGui.QMainWindow):
         self.ui.dateTimeEdit_expiration.setDateTime(self.model.expiration_date)
         self.ui.dateTimeEdit_expiration.setEnabled(True)
         
+        self.ui.action_optimize_wf.setEnabled(False)
         self.ui.action_submit.setEnabled(False)
         self.ui.action_change_expiration_date.setEnabled(True)
         self.ui.action_delete_workflow.setEnabled(True)
+        self.ui.action_stop_wf.setEnabled(True)
+        self.ui.action_restart.setEnabled(True)
         self.ui.action_transfer_infiles.setEnabled(True)
         self.ui.action_transfer_outfiles.setEnabled(True)    
         self.ui.action_save.setEnabled(True)    
@@ -724,7 +819,7 @@ class WorkflowWidget(QtGui.QMainWindow):
   @QtCore.pyqtSlot()  
   def refreshWorkflowList(self):
     self.updateWorkflowList()
-    self.model.clearCurrentWorkflow()
+    self.model.clear_current_workflow()
 
 
   def updateWorkflowList(self):
@@ -758,10 +853,10 @@ class WorkflowWidget(QtGui.QMainWindow):
     if answer == QtGui.QMessageBox.Yes:
       new_connection = self.createConnection(self.model.current_resource_id)
       if new_connection:
-        self.model.reinitCurrentConnection(new_connection)
+        self.model.reinit_current_connection(new_connection)
         return True
       else:
-        self.model.deleteCurrentConnection()
+        self.model.delete_current_connection()
         return False
     
 
@@ -1056,7 +1151,7 @@ class GroupInfoWidget(QtGui.QWidget):
     
   def dataChanged(self):
     
-    job_nb = len(self.group_item.not_sub) + len(self.group_item.done) + len(self.group_item.failed) + len(self.group_item.running)
+    job_nb = len(self.group_item.not_sub) + len(self.group_item.done) + len(self.group_item.failed) + len(self.group_item.running) + len(self.group_item.warning)
     
     ended_job_nb = len(self.group_item.done) + len(self.group_item.failed)
     
@@ -1608,22 +1703,78 @@ class WorkflowItemModel(QtCore.QAbstractItemModel):
 #######################   MODEL   ######################
 ########################################################
 
-class GuiModel(QtCore.QObject):
+
+class ComputingResourcePool(object):
   '''
-  Model for the GUI. This model was created to provide faster
-  GUI minimizing communications with the server.
+  Holds the instances of soma.workflow.client.WorkflowController associated 
+  to a unique resource id.
+  '''
+
+  # dictonary: resource id -> soma.workflow.client.WorkflowController
+  _connections = None
+
+  # dictionary: resource id -> lock
+  _connection_locks = None
+  
+  def __init__(self):
+    self._connections = {}
+    self._connection_locks = {}
+
+  def add_default_connection(self):
+    resource_id = socket.gethostname()
+    if resource_id not in self._connections.keys():
+      self.add_connection(resource_id, WorkflowController())
+
+  def add_connection(self, resource_id, workflow_controller):
+    self._connections[resource_id] = workflow_controller
+    self._connection_locks[resource_id] = threading.RLock()
+
+  def delete_connection(self, resource_id):
+    del self._connections[resource_id] 
+    del self._connection_locks[resource_id]
+
+  def reinit_connection(self, resource_id, workflow_controller):
+    with self._connection_locks[resource_id]:
+      self._connections[resource_id] = workflow_controller
+    
+  def connection(self, resource_id):
+    return self._connections[resource_id]
+
+  def lock(self, resource_id):
+    return self._connection_locks[resource_id]
+
+  def resource_exist(self, resource_id):
+    return resource_id in self._connections.keys()
+
+  def any_resource_id(self):
+    if len(self._connections) > 0:
+      return self._connections.keys()[0]
+    else:
+      return None
+
+
+
+class ApplicationModel(QtCore.QObject):
+  '''
+  Model for the application. This model was created to provide faster
+  application minimizing communications with the server.
   The instances of this class hold the connections and the GuiWorkflow instances 
-  created in the current session of the GUI.
-  The current workflow is periodically updated and the signal data_changed is
+  created in the current session of the application.
+  The current workflow is periodically updated and the signal workflow_state_changed is
   emitted if necessary.
   '''
   
-  def __init__(self, parent = None):
-    
-    super(GuiModel, self).__init__(parent)
+  def __init__(self, resource_pool=None, parent=None):
+    '''
+    **resouce_pool**: *ComputingResourcePool*
+    '''
+    super(ApplicationModel, self).__init__(parent)
 
+    if resource_pool != None:
+      self._resource_pool = resource_pool
+    else:
+      self._resource_pool = ComputingResourcePool()
     
-    self.connections = {} # ressource_id => connection
     self.workflows = {} # gui_workflows # resource_id => workflow_id => GuiWorkflow
     self.expiration_dates = {} # resource_id => workflow_ids => expiration_dates
         
@@ -1639,7 +1790,7 @@ class GuiModel(QtCore.QObject):
     
     self._lock = threading.RLock()
     
-    def updateLoop(self):
+    def update_loop(self):
       # update only the current workflows
       while True:
         with self._lock:
@@ -1654,28 +1805,49 @@ class GuiModel(QtCore.QObject):
                   #end = datetime.now() - begining
                   #print " <== end communication" + repr(self.wf_id) + " : " + repr(end.seconds)
               except ConnectionClosedError, e:
-                self.emit(QtCore.SIGNAL('connection_closed_error()'))
+                self.connection_closed_error()
+                #self.emit(QtCore.SIGNAL('connection_closed_error()'))
                 self.hold = True
               except UnknownObjectError, e:
                 self.delete_workflow()
               else: 
                 if self.current_workflow and self.current_workflow.updateState(wf_status): 
-                  self.emit(QtCore.SIGNAL('workflow_state_changed()'))
+                  self.workflow_state_changed()
+                  #self.emit(QtCore.SIGNAL('workflow_state_changed()'))
         time.sleep(self.update_interval)
     
-    self.__update_thread = threading.Thread(name = "GuiModelUpdateLoop",
-                                           target = updateLoop,
+    self.__update_thread = threading.Thread(name = "wf_execution_context_update_loop",
+                                           target = update_loop,
                                            args = ([self]))
     self.__update_thread.setDaemon(True)
     self.__update_thread.start()
    
+
+  def connection_closed_error(self):
+    print "====>  connection closed error"
+    self.emit(QtCore.SIGNAL('connection_closed_error()'))
+
+  def current_connection_changed(self):
+    self.emit(QtCore.SIGNAL('current_connection_changed()'))
+
+  def workflow_state_changed(self):
+    self.emit(QtCore.SIGNAL('workflow_state_changed()'))
+
+  def current_workflow_about_to_change(self):
+    self.emit(QtCore.SIGNAL('current_workflow_about_to_change()'))
+
+  def current_workflow_changed(self):
+    self.emit(QtCore.SIGNAL('current_workflow_changed()'))
    
-  def addConnection(self, resource_id, connection):
+  def resource_exist(self, resource_id):
+    return self._resource_pool.resource_exist(resource_id)
+
+  def add_connection(self, resource_id, connection):
     '''
     Adds a connection and use it as the current connection
     '''
     with self._lock:
-      self.connections[resource_id] = connection
+      self._resource_pool.add_connection(resource_id, connection)
       self.workflows[resource_id] = {}
       self.expiration_dates[resource_id] = {}
       self.current_resource_id = resource_id
@@ -1683,17 +1855,18 @@ class GuiModel(QtCore.QObject):
       self.current_workflow = None
       self.current_wf_id = -1
       self.expiration_date = None
-      self.emit(QtCore.SIGNAL('current_connection_changed()'))
+      self.current_connection_changed()
+      #self.emit(QtCore.SIGNAL('current_connection_changed()'))
       self.hold = False
 
-  def deleteCurrentConnection(self):
+  def delete_current_connection(self):
     '''
     Delete the current connection.
     If any other connections exist the new current connection will be one of them.
     If not the current connection is set to None.
     '''
     with self._lock:
-      del self.connections[self.current_resource_id] 
+      self._resource_pool.delete_connection(self.current_resource_id)
       del self.workflows[self.current_resource_id]
       del self.expiration_dates[self.current_resource_id]
       self.current_resource_id = None
@@ -1702,35 +1875,39 @@ class GuiModel(QtCore.QObject):
       self.current_wf_id = -1
       self.expiration_date = None
     
-      if len(self.connections.keys()):
-        self.current_resource_id = self.connections.keys()[0]
-        self.current_connection = self.connections[self.current_resource_id]
-      self.emit(QtCore.SIGNAL('current_connection_changed()'))
+      self.current_resource_id = self._resource_pool.any_resource_id()
+      if self.current_resource_id != None:
+        self.current_connection = self._resource_pool.connection(self.current_resource_id)
+
+      self.current_connection_changed()
+      #self.emit(QtCore.SIGNAL('current_connection_changed()'))
       self.hold = False
     
-  def setCurrentConnection(self, resource_id):
+  def set_current_connection(self, resource_id):
     with self._lock:
       if resource_id != self.current_resource_id:
-        assert(resource_id in self.connections.keys())
+        assert(self._resource_pool.resource_exist(resource_id))
         self.current_resource_id = resource_id
-        self.current_connection = self.connections[resource_id]
-        self.emit(QtCore.SIGNAL('current_connection_changed()'))
+        self.current_connection = self._resource_pool.connection(resource_id)
+        self.current_connection_changed()
+        #self.emit(QtCore.SIGNAL('current_connection_changed()'))
         self.hold = False
 
-  def reinitCurrentConnection(self, connection):
+  def reinit_current_connection(self, connection):
     with self._lock:
       self.current_connection = connection
-      self.connections[self.current_resource_id] = connection
+      self._resource_pool.reinit_connection(self.current_resource_id, connection)
       self.hold = False
     
-  def addWorkflow(self, workflow, expiration_date):
+  def add_workflow(self, workflow, expiration_date):
     '''
     Build a GuiWorkflow from a soma.workflow.client.Worklfow and 
     use it as the current workflow. 
     @type worklfow: soma.workflow.client.Workflow
     '''
     with self._lock:
-      self.emit(QtCore.SIGNAL('current_workflow_about_to_change()'))
+      self.current_workflow_about_to_change()
+      #self.emit(QtCore.SIGNAL('current_workflow_about_to_change()'))
       self.hold = True 
       self.current_workflow = GuiWorkflow(workflow)
       self.current_wf_id = self.current_workflow.wf_id
@@ -1741,53 +1918,80 @@ class GuiModel(QtCore.QObject):
         try:
           wf_status = self.current_connection.workflow_elements_status(self.current_workflow.wf_id)
         except ConnectionClosedError, e:
-          self.emit(QtCore.SIGNAL('connection_closed_error()'))
+          self.connection_closed_error()
+          #self.emit(QtCore.SIGNAL('connection_closed_error()'))
         else: 
           self.current_workflow.updateState(wf_status)
-      self.emit(QtCore.SIGNAL('current_workflow_changed()'))
+      self.current_workflow_changed()
+      #self.emit(QtCore.SIGNAL('current_workflow_changed()'))
       self.hold = False
 
-  def restartCurrentWorkflow(self):
+  def restart_current_workflow(self):
     self.current_workflow.restart()
     
   def delete_workflow(self):
     with self._lock:
-      self.emit(QtCore.SIGNAL('current_workflow_about_to_change()'))
+      self.current_workflow_about_to_change()
+      #self.emit(QtCore.SIGNAL('current_workflow_about_to_change()'))
       if self.current_workflow and self.current_workflow.wf_id in self.workflows.keys():
         del self.workflows[wf_id]
         del self.expiration_dates[wf_id]
       self.current_workflow = None
       self.current_wf_id = -1
       self.expiration_date = datetime.now()
+      self.current_workflow_changed()
+      #self.emit(QtCore.SIGNAL('current_workflow_changed()'))
     
-      self.emit(QtCore.SIGNAL('current_workflow_changed()'))
-    
-  def clearCurrentWorkflow(self):
+  def clear_current_workflow(self):
     with self._lock:
       if self.current_workflow != None or  self.current_wf_id != -1:
-        self.emit(QtCore.SIGNAL('current_workflow_about_to_change()'))
+        self.current_workflow_about_to_change()
+        #self.emit(QtCore.SIGNAL('current_workflow_about_to_change()'))
         self.current_workflow = None
         self.current_wf_id = -1
         self.expiration_date = datetime.now()
-        self.emit(QtCore.SIGNAL('current_workflow_changed()'))
+        self.current_workflow_changed()
+        #self.emit(QtCore.SIGNAL('current_workflow_changed()'))
     
-  def setCurrentWorkflow(self, wf_id):
+  def set_current_workflow(self, wf_id):
     with self._lock:
       if wf_id != self.current_wf_id:
         assert(wf_id in self.workflows[self.current_resource_id].keys())
-        self.emit(QtCore.SIGNAL('current_workflow_about_to_change()'))
+        self.current_workflow_about_to_change()
+        #self.emit(QtCore.SIGNAL('current_workflow_about_to_change()'))
         self.current_wf_id = wf_id
         self.current_workflow = self.workflows[self.current_resource_id][self.current_wf_id]
         self.expiration_date = self.expiration_dates[self.current_resource_id][self.current_wf_id]
-        self.emit(QtCore.SIGNAL('current_workflow_changed()'))
+        self.current_workflow_changed()
+        #self.emit(QtCore.SIGNAL('current_workflow_changed()'))
       
-  def changeExpirationDate(self, date):
+  def change_expiration_date(self, date):
      self.expiration_date = date 
      self.expiration_dates[self.current_resource_id][self.current_workflow.wf_id] = self.expiration_date
 
-  def isLoadedWorkflow(self, wf_id):
+  def is_loaded_workflow(self, wf_id):
     return wf_id in self.workflows[self.current_resource_id].keys()
   
+
+
+#class GuiModel(ApplicationModel, QtCore.QObject):
+  #def __init__(self, resource_pool=None, parent=None):
+    #super(GuiModel, self).__init__(parent=parent, resource_pool=resource_pool)
+
+  #def connection_closed_error(self):
+    #self.emit(QtCore.SIGNAL('connection_closed_error()'))
+
+  #def current_connection_changed(self):
+    #self.emit(QtCore.SIGNAL('current_connection_changed()'))
+
+  #def workflow_state_changed(self):
+    #self.emit(QtCore.SIGNAL('workflow_state_changed()'))
+
+  #def current_workflow_about_to_change(self):
+    #self.emit(QtCore.SIGNAL('current_workflow_about_to_change()'))
+
+  #def current_workflow_changed(self):
+    #self.emit(QtCore.SIGNAL('current_workflow_changed()'))
   
 class GuiWorkflow(object):
 
@@ -2073,6 +2277,7 @@ class GuiGroup(GuiWorkflowItem):
     self.done = []
     self.failed = []
     self.running = []
+    self.warning = []
     
     self.first_sub_date = None
     self.last_end_date = None
@@ -2100,9 +2305,9 @@ class GuiGroup(GuiWorkflowItem):
     self.done = []
     self.failed = []
     self.running = []
+    self.warning = []
 
     no_status = False
-    warning = False
     
     for child in self.children:
       item = self.gui_workflow.items[child]
@@ -2112,9 +2317,8 @@ class GuiGroup(GuiWorkflowItem):
           no_status = True
           break
         if item.status == WARNING:
-          warning = True
-          break
-        if item.status == NOT_SUBMITTED:
+          self.warning.append(item)
+        elif item.status == NOT_SUBMITTED:
           self.not_sub.append(item)
         elif item.status == DONE or item.status == FAILED:
           exit_status, exit_value, term_signal, resource_usage = item.exit_info
@@ -2139,6 +2343,7 @@ class GuiGroup(GuiWorkflowItem):
         self.done.extend(item.done)
         self.failed.extend(item.failed)
         self.running.extend(item.running)
+        self.warning.extend(item.warning)
         self.input_to_transfer.extend(item.input_to_transfer)
         self.input_transfer_ended.extend(item.input_transfer_ended)
         self.output_ready.extend(item.output_ready)
@@ -2151,7 +2356,7 @@ class GuiGroup(GuiWorkflowItem):
            
     if no_status:
       new_status = GuiGroup.GP_NO_STATUS
-    elif warning:
+    elif len(self.warning) > 0:
       new_status = GuiGroup.GP_WARNING
     elif len(self.failed) > 0:
       new_status = GuiGroup.GP_FAILED
@@ -2306,7 +2511,10 @@ class GuiTransfer(GuiWorkflowItem):
       elif len(transfer_status_info[1]) == 3:
         self.transfer_type = GuiTransfer.DIRECTORY
         (size, transmitted, elements_status) = transfer_status_info[1]
-      self.percentage_achievement = int(float(transmitted)/size * 100.0)
+      if size != 0:
+        self.percentage_achievement = int(float(transmitted)/size * 100.0)
+      else:
+        self.percentage_achievement = 100
     else:
       (size, transmitted, elements_status) = (None, None, None)
       self.percentage_achievement = 0
