@@ -1,3 +1,5 @@
+#!/usr/bin/env python
+
 '''
 @author: Soizic Laguitton
 @organization: U{IFR 49<http://www.ifr49.org>}
@@ -11,12 +13,14 @@ if __name__=="__main__":
   import threading
   import time 
   import logging
+  import os
 
   import Pyro.naming
   import Pyro.core
   from Pyro.errors import PyroError, NamingError
   
   import soma.workflow.engine
+  import soma.workflow.scheduler
   import soma.workflow.connection 
   from soma.workflow.configuration import Configuration
   from soma.workflow.errors import EngineError
@@ -50,7 +54,7 @@ if __name__=="__main__":
    engine_log_format,
    engine_log_level) = config.get_engine_log_info()
    if engine_log_dir:
-    logfilepath = engine_log_dir + "log_" + engine_name + log
+    logfilepath = os.path.join(os.path.abspath(engine_log_dir),  "log_" + engine_name + log)
     logging.basicConfig(
           filename=logfilepath,
           format=engine_log_format,
@@ -89,7 +93,7 @@ if __name__=="__main__":
     Pyro.core.initServer()
     daemon = Pyro.core.Daemon()
 
-    drmaa = soma.workflow.engine.Drmaa(config.get_drmaa_implementation(), 
+    drmaa = soma.workflow.scheduler.Drmaa(config.get_drmaa_implementation(), 
                                        config.get_parallel_job_config())
 
     engine_loop = soma.workflow.engine.WorkflowEngineLoop(database_server,
